<?php
/**
 * Handles Authorization
 *
 * @package embed-sharepoint-onedrive-documents/API
 */

namespace MoSharePointObjectSync\API;

use MoSharePointObjectSync\Wrappers\wpWrapper;

/**
 * Class to handle token authorization and API endpoints' requests
 */
class Authorization {

	/**
	 * Holds the Authorization class instance.
	 *
	 * @var Authorization
	 */
	private static $instance;

	/**
	 * Object instance(Authorization) getter method.
	 *
	 * @return Authorization
	 */
	public static function get_controller() {
		if ( ! isset( self::$instance ) ) {
			$class          = __CLASS__;
			self::$instance = new $class();
		}
		return self::$instance;
	}

	/**
	 * Function to get csom access token using client credentials grant type.
	 *
	 * @param array  $endpoints This holds array of all the endpoints of Outlook REST APIs.
	 * @param array  $config This holds array of azure application client credentials.
	 * @param string $scope This is vaue of scope to be passed in token endpoint.
	 * @return array
	 */
	public function mo_sps_get_access_token_using_client_credentials( $endpoints, $config, $scope ) {
		if ( ! empty( $config ) ) {
			$client_secret = wpWrapper::mo_sps_decrypt_data( $config['client_secret'], hash( 'sha256', $config['client_id'] ) );
			$url           = $config['admin_uri'];
			$domain        = wpWrapper::mo_sps_get_domain_from_url( $url );
			$args = array(
				'body'    => array(
					'grant_type'    => 'client_credentials',
					'client_secret' => $client_secret,
					'client_id'     => $config['client_id'] . '@' . $config['tenant_id'],
					'resource'      => '00000003-0000-0ff1-ce00-000000000000/' . $domain . '@' . $config['tenant_id'],
				),
				'headers' => array(
					'Content-type' => 'application/x-www-form-urlencoded',
				),

			);

			$response = wp_remote_post( esc_url_raw( $endpoints['token'] ), $args );

			if ( is_wp_error( $response ) ) {
				return array(
					'status' => false,
					'data'   => array(
						'error'             => 'Request timeout',
						'error_description' => 'Unexpected error occurred! Please check your internet connection and try again.',
					),
				);
			} else {
				$body = json_decode( $response['body'], true );
				if ( isset( $body['access_token'] ) ) {
					return array(
						'status' => true,
						'data'   => $body['access_token'],
					);
				} elseif ( isset( $body['error'] ) ) {
					return array(
						'status' => false,
						'data'   => $body,
					);
				}
			}
		}

		return array(
			'status' => false,
			'data'   => array(
				'error'             => 'Unexpected Error',
				'error_description' => 'Check your configurations once again',
			),
		);
	}

	/**
	 * Function to get graph access token using client credentials grant type.
	 *
	 * @param array  $endpoints This holds array of all the endpoints of Outlook REST APIs.
	 * @param array  $config This holds array of azure application client credentials.
	 * @param string $scope This is vaue of scope to be passed in token endpoint.
	 * @return array
	 */
	public function mo_sps_get_grpah_access_token_using_client_credentials( $endpoints, $config, $scope ) {
		if ( ! empty( $config ) ) {
			$client_secret = wpWrapper::mo_sps_decrypt_data( $config['client_secret'], hash( 'sha256', $config['client_id'] ) );

			$args = array(
				'body'    => array(
					'grant_type'    => 'client_credentials',
					'client_secret' => $client_secret,
					'client_id'     => $config['client_id'],
					'scope'         => $scope,
				),
				'headers' => array(
					'Content-type' => 'application/x-www-form-urlencoded',
				),

			);

			$response = wp_remote_post( esc_url_raw( $endpoints['graph_token'] ), $args );
			if ( is_wp_error( $response ) ) {
				return array(
					'status' => false,
					'data'   => array(
						'error'             => 'Request timeout',
						'error_description' => 'Unexpected error occurred! Please check your internet connection and try again.',
					),
				);
			} else {
				$body = json_decode( $response['body'], true );
				if ( isset( $body['access_token'] ) ) {
					return array(
						'status' => true,
						'data'   => $body['access_token'],
					);
				} elseif ( isset( $body['error'] ) ) {
					return array(
						'status' => false,
						'data'   => $body,
					);
				}
			}
		}
		return array(
			'status' => false,
			'data'   => array(
				'error'             => 'Unexpected Error',
				'error_description' => 'Check your configurations once again',
			),
		);
	}

	/**
	 * Function to execute API calls using GET method.
	 *
	 * @param string  $url This contains api endpoint where GET method should be carried out.
	 * @param array   $headers This contains array of headers that to be passed in API call.
	 * @param boolean $check To decide whether to escape api endpoint or not.
	 * @return array
	 */
	public function mo_sps_get_request( $url, $headers, $check ) {
		$args = array(
			'headers' => $headers,
		);
		if ( $check ) {
			$response = wp_remote_get( esc_url_raw( $url ), $args );
		} else {
			$response = wp_remote_get( $url, $args );
		}
		if ( is_array( $response ) && ! is_wp_error( $response ) ) {
			if ( 'Forbidden' === $response['response']['message'] ) {
				return $response['response']['message'];
			} else {
				return json_decode( $response['body'], true );
			}
		} else {
			wp_die( 'Error occurred: ' . esc_html( $response->get_error_message() ) );
		}
	}

}
