<?php
/**
 * Handles SharePoint CSOM APIs.
 *
 * @package embed-sharepoint-onedrive-documents/API
 */

namespace MoSharePointObjectSync\API;

use MoSharePointObjectSync\Wrappers\pluginConstants;
use MoSharePointObjectSync\Wrappers\wpWrapper;

/**
 * Class to handle all SharePoint API endpoints
 */
class Azure {

	/**
	 * Holds the Azure class instance.
	 *
	 * @var Azure
	 */
	private static $obj;

	/**
	 * Array of all SharePoint endpoints.
	 *
	 * @var array
	 */
	private $endpoints;

	/**
	 * Array of all Azure application configurations like client ID & secret
	 *
	 * @var array
	 */
	private $config;

	/**
	 * Scope value that should be passed while requesting for token.
	 *
	 * @var string
	 */
	private $scope = 'https://graph.microsoft.com/.default';

	/**
	 * It holds access token value.
	 *
	 * @var string
	 */
	private $access_token;

	/**
	 * Holds the Authorization class instance.
	 *
	 * @var Authorization
	 */
	private $handler;

	/**
	 * Contructor of Azure class.
	 *
	 * @param array $config This contains azure ad client credentials.
	 */
	private function __construct( $config ) {
		$this->config  = $config;
		$this->handler = Authorization::get_controller();
	}


	/**
	 * Object instance(Azure) getter method.
	 *
	 * @param array $config This contains azure ad client credentials.
	 * @return Azure
	 */
	public static function get_client( $config ) {
		if ( ! isset( self::$obj ) ) {

			self::$obj = new Azure( $config );

			self::$obj->set_endpoints();

		}
		return self::$obj;
	}

	/**
	 * It is used to set the endpoints of SharePoint/Azure APIs.
	 *
	 * @return void
	 */
	private function set_endpoints() {
		if ( ! empty( $this->config ) ) {
			$this->endpoints['token']       = 'https://accounts.accesscontrol.windows.net/' . $this->config['tenant_id'] . '/tokens/OAuth/2';
			$this->endpoints['graph_token'] = 'https://login.microsoftonline.com/' . $this->config['tenant_id'] . '/oauth2/v2.0/token';
		}
	}

	/**
	 * Function to get new CSOM access token
	 *
	 * @return mixed
	 */
	public function mo_sps_get_new_csom_access_token() {

		$response = $this->handler->mo_sps_get_access_token_using_client_credentials( $this->endpoints, $this->config, $this->scope );

		if ( $response['status'] ) {

			$this->access_token = $response['data'];

			return $this->access_token;
		}

		$this->access_token = false;

		return false;
	}

	/**
	 * Function to test connection of configured app.
	 *
	 * @return array
	 */
	public function mo_sps_test_azure_ad_connection() {

		$response = $this->handler->mo_sps_get_grpah_access_token_using_client_credentials( $this->endpoints, $this->config, $this->scope );
		return $response;
	}

	/**
	 * Function to test connection of sharepoint urls.
	 *
	 * @return array
	 */
	public function mo_sps_test_sharepoint_connection() {

		$response = $this->handler->mo_sps_get_access_token_using_client_credentials( $this->endpoints, $this->config, $this->scope );
		return $response;
	}

	/**
	 * Function to get document details based on server relative path.
	 *
	 * @param string $server_relative_url // server relative path of document.
	 * @return mixed
	 */
	public function mo_sps_document_array( $server_relative_url ) {
		$app = wpWrapper::mo_sps_get_option( pluginConstants::APP_CONFIG );
		$this->mo_sps_get_new_csom_access_token();

		$urlarray = explode( '/', $app['site_uri'] );
		$end      = $urlarray[ count( $urlarray ) - 1 ];
		if ( '' !== $app['site_uri'] ) {

			$second_last = $urlarray[ count( $urlarray ) - 2 ];
		}

		$args = array(
			'Authorization' => 'Bearer ' . $this->access_token,
			'Accept'        => 'application/json; odata=verbose',
		);

		if ( ( ! empty( $app['site_uri'] ) && 'https://' . wpWrapper::mo_sps_get_domain_from_url( $app['site_uri'] ) === 'https://' . wpWrapper::mo_sps_get_domain_from_url( $app['admin_uri'] ) ) || empty( $app['site_uri'] ) ) {
				$server_relative_url = $server_relative_url;
		} else {

			$server_relative_url = '/' . $second_last . '/' . $end . $server_relative_url;

		}

		if ( ! empty( $app['admin_uri'] ) ) {
			$server_relative_url = wpWrapper::mo_urlencode( $server_relative_url );

			$url = ! empty( $app['site_uri'] ) ? 'https://' . wpWrapper::mo_sps_get_domain_from_url( $app['site_uri'] ) . "/_api/web/GetFolderByServerRelativePath(decodedUrl='$server_relative_url')/?" . '$expand=files,folders&$expand=Editor/Id' : 'https://' . wpWrapper::mo_sps_get_domain_from_url( $app['admin_uri'] ) . "/_api/web/GetFolderByServerRelativePath(decodedUrl='$server_relative_url')/?" . '$expand=files,folders&$expand=Editor/Id';

			$response = $this->handler->mo_sps_get_request( $url, $args, true );

		} else {
			$response = '';
		}

		return $response;
	}

	/**
	 * Function to search documents by keyword
	 *
	 * @param string $query_text // search keyword.
	 * @return mixed
	 */
	public function mo_sps_document_search( $query_text ) {
		$app = wpWrapper::mo_sps_get_option( pluginConstants::APP_CONFIG );
		$this->mo_sps_get_new_csom_access_token();

		$urlarray = explode( '/', $app['site_uri'] );
		$end      = $urlarray[ count( $urlarray ) - 1 ];

		$args = array(
			'Authorization' => 'Bearer ' . $this->access_token,
			'Accept'        => 'application/json; odata=verbose',
		);

		if ( ( ! empty( $app['site_uri'] ) && 'https://' . wpWrapper::mo_sps_get_domain_from_url( $app['site_uri'] ) === 'https://' . wpWrapper::mo_sps_get_domain_from_url( $app['admin_uri'] ) ) || empty( $app['site_uri'] ) ) {
			if ( ! empty( $app['admin_uri'] ) ) {

				$url = ! empty( $app['site_uri'] ) ? 'https://' . wpWrapper::mo_sps_get_domain_from_url( $app['site_uri'] ) . '/_api/search/query?querytext=' . $query_text : 'https://' . wpWrapper::mo_sps_get_domain_from_url( $app['admin_uri'] ) . '/_api/search/query?querytext=' . $query_text;

				$response = $this->handler->mo_sps_get_request( $url, $args, false );
			} else {
				$response = '';
			}
				return $response;
		} else {

			$url = ! empty( $app['site_uri'] ) ? 'https://' . wpWrapper::mo_sps_get_domain_from_url( $app['site_uri'] ) . '/_api/search/query?querytext=' . $query_text : 'https://' . wpWrapper::mo_sps_get_domain_from_url( $app['admin_uri'] ) . '/_api/search/query?querytext=' . $query_text;

				$response = $this->handler->mo_sps_get_request( $url, $args, false );
				return $response;
		}
	}
}
